/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "gctl/io.h"
#include "../lib/potential.h"

using namespace gctl;

int main(int argc, char const *argv[])
{
	array<vertex3dc> node;
	array<mag_tetrahedron> ele;

	gmshio gio;
	gio.init_file("data/pie/pie", Input);
	gio.set_packed(NotPacked, Input);
	gio.read_mesh(ele, node);

	array<magtet_para> mag_para;
	array<double> sus(ele.size(), 40.0);
	callink_magnetic_para_earth(ele, mag_para, 90.0, 0.0);

	// 设置观测点位
	array<point3dc> obsp;
	gridspace(point3dc(-20, 0, 5), point3dc(20, 0, 5), point3dc(0, -20, 5), point3dc(0, 20, 5), 201, 201, obsp);

	// 正演计算
	array<double> obsval(obsp.size());
	array<point3dc> obsgrad;

	// 正演磁分量数据
	magobser(obsgrad, ele, obsp, sus, ShortMsg);
	// 保存网格
	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].x;
	}
	gctl::save_netcdf_grid("out_magobser_tetra", obsval, 201, 201, -20.0, 0.2, -20.0, 0.2, "x", "y", "Bx");

	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].y;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "By");

	for (int i = 0; i < obsp.size(); ++i)
	{
		obsval[i] = obsgrad[i].z;
	}
	gctl::append_netcdf_grid("out_magobser_tetra", obsval, "x", "y", "Bz");

	return 0;
}