﻿/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GM_REGULAR_MESH_3D_H
#define _GM_REGULAR_MESH_3D_H

#include "gctl/mesh/regular_mesh_3d.h"

#include "gkernel_block.h"
#include "mkernel_block.h"

namespace gctl
{
	class gm_regular_mesh_3d : public regular_mesh_3d
	{
	protected:
		array<mag_block> mag_ele_;

	public:
		gm_regular_mesh_3d();
		gm_regular_mesh_3d(std::string in_name, std::string in_info, 
			int xbnum, int ybnum, int zbnum, double xmin, double ymin, double zmin, 
			double xsize, double ysize, double zsize);
		virtual ~gm_regular_mesh_3d();

		void gkernel(matrix<double> &out_kernel, const array<point3dc> &obs, 
			gravitational_field_type_e comp_id = Vz, verbose_type_e verbose = FullMsg);
		void gobser(array<double> &out_data, std::string data_name, const array<point3dc> &obs, 
			gravitational_field_type_e comp_id = Vz, verbose_type_e verbose = FullMsg);
        void magkernel(matrix<double> &out_kernel, const array<point3dc> &obs, 
            const array<double> &inc_deg, const array<double> &dec_deg, double geo_inc, double geo_dec, 
            magnetic_field_type_e comp_id = DeltaT, verbose_type_e verbose = FullMsg);
        void magkernel(matrix<double> &out_kernel, const array<point3dc> &obs, 
            const array<double> &inc_deg, const array<double> &dec_deg, 
            magnetic_field_type_e comp_id = Za, verbose_type_e verbose = FullMsg);
        void magobser(array<double> &out_data, std::string data_name, const array<point3dc> &obs, 
            const array<double> &inc_deg, const array<double> &dec_deg, double geo_inc, double geo_dec, 
            magnetic_field_type_e comp_id = DeltaT, verbose_type_e verbose = FullMsg);
        void magobser(array<double> &out_data, std::string data_name, const array<point3dc> &obs, 
            const array<double> &inc_deg, const array<double> &dec_deg, 
            magnetic_field_type_e comp_id = Za, verbose_type_e verbose = FullMsg);
	};
}

#endif //_GM_REGULAR_MESH_3D_H